XMAVLink.Util.Context (xmavlink v0.14.3)

Copy Markdown View Source

Utility-layer runtime context.

Utility helpers use this context to choose the router, dialect, and ETS table namespace they operate on. Pass context: context to utility functions when an application has more than one MAVLink runtime or when global ETS table names are not acceptable.

Summary

Types

t()

@type t() :: %XMAVLink.Util.Context{
  dialect: module(),
  router: GenServer.server(),
  table_prefix: table_prefix(),
  tables: %{required(atom()) => atom()}
}

table_prefix()

@type table_prefix() :: atom() | String.t() | nil

Functions

new(opts \\ [])

@spec new(keyword() | map() | t()) :: t()