XMAVLink.Util.FocusManager (xmavlink v0.14.3)

Copy Markdown View Source

Manage a protected ETS table representing the nominated MAV focus of zero or more local PIDs. The API uses this to streamline iex sessions by letting the user select a MAV to work with and transparently adding {system_id, component_id} tuples to call arguments.

Pass context: context to read or write focus in a scoped utility table namespace.

Summary

Functions

child_spec(init_arg)

Returns a specification to start this module under a supervisor.

See Supervisor.

focus()

focus(opts)

focus(pid, opts)

focus(system_id, component_id, opts)

start_link(state, opts \\ [])